/**********************************************************************
 * Copyright (c) 2010, 2016 IBM Corporation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.trace.ui;

import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.hyades.trace.ui.internal.core.TraceProfileFiltersUI;
import org.eclipse.hyades.trace.ui.internal.launcher.ProfileTab;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

public interface IProfilerUIAdditionExtension {

	/** Provides extensions the parent composite of the profiling tab, the current launch configuration,
	 * and a reference to the profile tab from which the composite is coming from. (Bug 323330) */
	public void addExtensionUIElements(Composite c, ILaunchConfiguration lc, ProfileTab profileTab);
	
	/**
	 * Provide extensions the parent composite of the TraceProfileFilterUI, to create a Customize auto 
	 * filtering criteria button to display workspace filter. 
	 */
	public void addExtensionUIDisplayButton(Composite composite,
			TraceProfileFiltersUI TPF);
	
	/**
	 * Provide extensions the parent composite of the TraceProfileFilterUI, to enable
	 * or disable the customize button based on the launchconfiguation. 
	 */
	public void enableAutoDisplayButton(final TraceProfileFiltersUI TPF);
	
	/**
	 * Provide extensions the parent composite of the TraceProfileFilterUI, to enable
	 * or disable functional buttons on filterUI based on the launchconfiguation. 
	 */
	public void applyAutoAffect(TraceProfileFiltersUI traceProfileFiltersUI);
	
	/**
	 * Provide extensions the parent composite of the TraceProfileFilterUI, to
	 * add additional checkbox indicating the option to automatically determine filter criteria 
	 */
	public void addExtensionUIAutoFilter(Composite composite,
			TraceProfileFiltersUI tPF);
	/**
	 * Provide extensions the parent composite of the TraceProfileFilterUI, to
	 * change text label on buttons of filterUI
	 */
	public void modifiedButtonLabel(Button btn, String newName);
	
	/**
	 * Provide extensions the parent composite of the TraceProfileFilterUI, to
	 * save auto-populate information in launch configuration
	 */
	public void storeAutoAttribute(ILaunchConfigurationWorkingCopy workingCopy,
			TraceProfileFiltersUI filterUI);
}
