/*******************************************************************************
 * Copyright (c) 2011 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: 
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.hyades.trace.ui.internal.export;

import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;

import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCFilter;
import org.eclipse.hyades.models.hierarchy.TRCNode;
import org.eclipse.hyades.models.hierarchy.TRCOption;
import org.eclipse.hyades.models.trace.TRCAggregatedMethodInvocation;
import org.eclipse.hyades.models.trace.TRCClass;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCFullTraceObject;
import org.eclipse.hyades.models.trace.TRCGCEvent;
import org.eclipse.hyades.models.trace.TRCMethod;
import org.eclipse.hyades.models.trace.TRCObjAllocSite;
import org.eclipse.hyades.models.trace.TRCProcess;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.models.trace.TRCThreadDeadEvent;
import org.eclipse.hyades.models.trace.TRCThreadRunningEvent;

public class XMLElementOutput implements ITraceElementOutput {

	final private static ITraceElementOutput _instance = new XMLElementOutput();

	final private static boolean DEBUG_TRACE_ORDER = false;
	
	private XMLElementOutput() {
	}
	
	public static ITraceElementOutput getInstance() {
		return _instance;
	}
	
	private static String pd1(double d) {
		BigDecimal de = new BigDecimal(d);
		
		String r = "";
		
		String str = ""+de.toPlainString();
		
		if(!(str.contains("."))) return str;
		
		int dot = str.indexOf(".");
		
		r += str.substring(0, dot) + ".";
		
		String afterDot = str.substring(dot+1); 
		
		r += afterDot.substring(0, Math.min(afterDot.length(), 9));
		
		return r;

	}


	private static String ad(double d, double startTime) {
		return ad(d, new BigDecimal(startTime));
	}
	
	private static String ad(double d, BigDecimal startTime) {
		String result = startTime.add(new BigDecimal(d)).toPlainString();
		
		int dot = result.indexOf(".");
		if(dot == -1) return result;
		
		String after = result.substring(dot+1);
		after = after.substring(0, Math.min(after.length(), 9));
		
		result = result.substring(0, dot)+"."+after;
		
		return result;
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.trace.TRCThreadRunningEvent)
	 */
	public void output(OutputStream os, TRCThreadRunningEvent e) throws IOException {
		TRCThread t = e.getThread();
		StringBuffer sb = new StringBuffer();
		
		sb.append("<threadStart ");
		sb.append("threadId=\""+t.getId()+"\" ");
		sb.append("time=\""+ad(e.getTime(), t.getProcess().getAbsoluteTime())+"\" ");
		sb.append("threadName=\""+t.getName()+"\" ");
		sb.append("groupName=\""+t.getGroupName()+"\" ");
		sb.append("parentName=\""+""+"\"/>");
		
		if(DEBUG_TRACE_ORDER) sb.append(e.getTraceOrder());		
		
		sb.append("\n");
		os.write(sb.toString().getBytes());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.trace.TRCThreadDeadEvent)
	 */
	public void output(OutputStream os, TRCThreadDeadEvent e) throws IOException {
		TRCThread t = e.getThread();
		StringBuffer sb = new StringBuffer();
		
		sb.append("<threadEnd ");
		sb.append("threadIdRef=\""+t.getId()+"\" ");
		sb.append("time=\""+ad(e.getTime(), t.getProcess().getAbsoluteTime())+"\"/>");
		if(DEBUG_TRACE_ORDER) sb.append(e.getTraceOrder());
		sb.append("\n");
		
		os.write(sb.toString().getBytes());
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.trace.TRCFullMethodInvocation, java.math.BigDecimal, boolean)
	 */
	public void output(OutputStream os, TRCFullMethodInvocation e, double d, BigDecimal s, boolean isEntry)  throws IOException {
		StringBuffer sb = new StringBuffer();
		
		if(isEntry) {
			sb.append("<methodEntry ");
			sb.append("threadIdRef=\""+e.getThread().getId()+"\" ");
//			sb.append("time=\""+pd1(e.getOrigEntryTime())+"\" ");
			sb.append("time=\""+ad(e.getEntryTime(), s)+"\" ");
			sb.append("methodIdRef=\""+e.getMethod().getId()+"\" ");
			sb.append("classIdRef=\""+e.getMethod().getDefiningClass().getId()+"\" ");
			sb.append("ticket=\""+e.getTicket()+"\" ");
			sb.append("stackDepth=\""+e.getStackDepth()+"\"/>");
			if(DEBUG_TRACE_ORDER) sb.append(e.getMethodEnterTraceOrder());

//			sb.append(" compare orig - time=\""+pd1(e.getOrigEntryTime())+"\" ");
//			sb.append(" - new - time=\""+ad(e.getEntryTime(), s)+"\" ");

			sb.append("\n");
		} else {
			sb.append("<methodExit ");
			sb.append("threadIdRef=\""+e.getThread().getId()+"\" ");
			sb.append("methodIdRef=\""+e.getMethod().getId()+"\" ");
			sb.append("classIdRef=\""+e.getMethod().getDefiningClass().getId()+"\" ");
			sb.append("ticket=\""+e.getTicket()+"\" ");
			
//			sb.append("time=\""+pd1(e.getOrigExitTime())+"\"/>");
			sb.append("time=\""+ad(e.getExitTime(), s)+"\"/>");
			if(DEBUG_TRACE_ORDER) sb.append(e.getMethodExitTraceOrder());

//			sb.append(" compare orig - time=\""+pd1(e.getOrigExitTime())+"\" ");
//			sb.append(" - new - time=\""+ad(e.getExitTime(), s)+"\" ");

			sb.append("\n");
		}
		
		os.write(sb.toString().getBytes());
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.trace.TRCAggregatedMethodInvocation, boolean)
	 */
	public void output(OutputStream os, TRCAggregatedMethodInvocation e, boolean isEntry)  throws IOException {
		StringBuffer sb = new StringBuffer();
		
		if(isEntry) {
		
			sb.append("<agMethodEntry ");
			sb.append("threadIdRef=\""+e.getThread().getId()+"\" ");
			sb.append("methodIdRef=\""+e.getMethod().getId()+"\" ");
			sb.append("baseTime=\""+pd1(e.getBaseTime())+"\" ");
			sb.append("minTime=\""+pd1(e.getMinTime())+"\" ");
			sb.append("maxTime=\""+pd1(e.getMaxTime())+"\" ");
			sb.append("count=\""+e.getCount()+"\"/>");
			if(DEBUG_TRACE_ORDER) sb.append(e.getMethodEnterTraceOrder());
			sb.append("\n");

		} else {
			
			sb.append("<agMethodExit ");
			sb.append("threadIdRef=\""+e.getThread().getId()+"\" ");
			sb.append("methodIdRef=\""+e.getMethod().getId()+"\"/>");
			if(DEBUG_TRACE_ORDER) sb.append(e.getMethodExitTraceOrder());
			sb.append("\n");
		}

		os.write(sb.toString().getBytes());
		
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#outputTraceEndTags(org.eclipse.hyades.models.trace.TRCProcess, java.math.BigDecimal, java.io.OutputStream)
	 */
	public void outputTraceEndTags(TRCProcess process, BigDecimal s, OutputStream os) throws IOException {
		StringBuffer sb = new StringBuffer();
		
		sb.append("<runtimeShutdown time=\""+ad(process.getStopTime(), s)+"\"/>");
		sb.append("\n");
		
		sb.append("<traceEnd time=\""+ad(process.getStopTime(),s)+"\"/>");
		sb.append("\n");
		
		sb.append("<agentDestroy ");
		sb.append("agentIdRef=\""+""+"\" ");
		sb.append("time=\""+ad(process.getStopTime(), s)+"\"/>");
		sb.append("\n");		
		
		sb.append("</TRACE>\n");
		
		os.write(sb.toString().getBytes());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#outputTraceStartTags(java.io.OutputStream, org.eclipse.hyades.models.trace.TRCProcess, org.eclipse.hyades.models.hierarchy.TRCAgentProxy)
	 */
	public void outputTraceStartTags(OutputStream out, TRCProcess process, TRCAgentProxy ap) throws IOException {
		StringBuffer sb = new StringBuffer();
		
		TRCNode node = ap.getProcessProxy().getNode();

		sb.append("<?xml version=\"1.0\"?>\n");
		sb.append("<TRACE>\n");

		String nodeTime = node.getAbsoluteTime() == null ? process.getAbsoluteTime().toPlainString() : node.getAbsoluteTime().toPlainString();
		
		// Create note
		sb.append("<node ");
		sb.append("nodeId=\""+""+"\" ");
		sb.append("hostname=\""+node.getName()+"\" ");
		String ipAddr = node.getIpAddress();
		if(ipAddr == null) ipAddr = "";
		sb.append("ipaddress=\""+ipAddr+"\" ");
		sb.append("timezone=\""+"0"+"\" ");
		sb.append("time=\""+(nodeTime)+"\"/>");
		sb.append("\n");
		
		sb.append("<processCreate ");
		sb.append("processId=\""+ap.getProcessProxy().getRuntimeId()+"\" ");
		sb.append("pid=\""+ap.getProcessProxy().getPid()+"\" ");
		sb.append("nodeIdRef=\""+""+"\" ");
		sb.append("time=\""+nodeTime+"\"/>");
		sb.append("\n");
		
		sb.append("<agentCreate ");
		sb.append("agentId=\""+ap.getRuntimeId()+"\" ");
		sb.append("version=\""+""+"\" ");
		sb.append("processIdRef=\""+""+"\" ");
		sb.append("agentName=\""+ap.getName()+"\" ");
		sb.append("agentType=\""+ap.getType()+"\" ");
		sb.append("agentParameters=\""+""+"\" ");
		sb.append("time=\""+nodeTime+"\"/>");
		sb.append("\n");
		
		sb.append("<traceStart ");
		sb.append("traceId=\""+process.getId()+"\" ");
		sb.append("agentIdRef=\""+""+"\" ");
		sb.append("time=\""+process.getAbsoluteTime().toPlainString()+"\"/>");
		sb.append("\n");

		out.write(sb.toString().getBytes());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.hierarchy.TRCFilter)
	 */
	public void output(OutputStream os, TRCFilter f) throws IOException {
		StringBuffer sb = new StringBuffer();
		
		sb.append("<filter ");
		if(f.getOrigPattern() != null) sb.append("pattern=\""+f.getOrigPattern()+"\" ");
		if(f.getOrigMode() != null) sb.append("mode=\""+f.getOrigMode()+"\" ");
		if(f.getOrigGenericPattern() != null) sb.append("genericPattern=\""+f.getOrigGenericPattern()+"\" ");
		if(f.getOrigMethodPattern() != null) sb.append("methodPattern=\""+f.getOrigMethodPattern()+"\" ");
		if(f.getOrigMethodMode() != null) sb.append("methodMode=\""+f.getOrigMethodMode()+"\" ");
		if(f.getOrigMethodGenericPattern() != null) sb.append("methodGenericPattern=\""+f.getOrigMethodGenericPattern()+"\"");
		sb.append("/>\n");

		os.write(sb.toString().getBytes());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.hierarchy.TRCOption)
	 */
	public void output(OutputStream os, TRCOption o) throws IOException {
		StringBuffer sb = new StringBuffer();
		sb.append("<option key=\""+o.getKey()+"\" value=\""+o.getValue()+"\"/>\n");
		os.write(sb.toString().getBytes());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#outputClassDef(org.eclipse.hyades.models.trace.TRCClass, org.eclipse.hyades.models.trace.TRCProcess, java.io.OutputStream)
	 */
	public void outputClassDef(TRCClass c, TRCProcess process, OutputStream os) throws IOException {
		StringBuffer sb = new StringBuffer();

		if(c.getName() == null || c.getId() < 0) return;
		
		sb.append("<classDef name=\""+c.getOrigFullName()+"\" ");
		if(c.getSourceInfo() != null && c.getSourceInfo().getLocation() != null) {
			String loct = c.getSourceInfo().getLocation();
			loct = loct.replace("<", "&lt;");
			loct = loct.replace(">", "&gt;");
			sb.append("sourceName=\""+loct+"\" ");
		}
		sb.append("classId=\""+c.getId()+"\" ");
		sb.append("time=\""+ad(c.getLoadTime(), process.getAbsoluteTime())+"\"/>\n");
		
		os.write(sb.toString().getBytes());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.trace.TRCMethod)
	 */
	public void output(OutputStream os, TRCMethod m) throws IOException {
		StringBuffer sb = new StringBuffer();
		
		sb.append("<methodDef ");
		sb.append("name=\""+m.getOrigName()+"\" " );
		sb.append("signature=\""+m.getOrigSignature()+"\" ");
		sb.append("startLineNumber=\""+m.getLineNo()+"\" ");
		sb.append("endLineNumber=\""+(m.getLineCount()+m.getLineNo())+"\" ");
		sb.append("methodId=\""+m.getId()+"\" ");
		sb.append("classIdRef=\""+m.getDefiningClass().getId()+"\"");
		sb.append("/>");
		sb.append("\n");

		os.write(sb.toString().getBytes());
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.trace.TRCGCEvent)
	 */
	public void output(OutputStream os, TRCGCEvent e) throws IOException {
		double initTime = e.getOwnerThread().getProcess().getAgent().getStartTime();
		
		StringBuffer sb = new StringBuffer();
		if (e.getType().equals("start")) {
			sb.append("<gcStart ");
		} else if(e.getType().equals("finish")) {
			sb.append("<gcFinish ");
		} else {
			return;
		}
		
		sb.append("time=\""+ad(e.getTime(), initTime)+"\"/>");
		sb.append("\n");
		os.write(sb.toString().getBytes());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.tptp.platform.common.ui.trace.internal.actions.ITraceElementOutput#output(java.io.OutputStream, org.eclipse.hyades.models.trace.TRCFullTraceObject, boolean)
	 */
	public void output(OutputStream os, TRCFullTraceObject o, boolean isAlloc) throws IOException {
		StringBuffer sb = new StringBuffer();
		double initTime = o.getProcess().getAgent().getStartTime();
		
		if(o.getSize() == 0) return;
		
		if(isAlloc) {
			sb.append("<objAlloc ");
			sb.append("threadIdRef=\""+o.getThread().getId()+"\" ");
			sb.append("time=\""+ad(o.getCreateTime(), initTime)+"\" ");
			sb.append("objId=\""+o.getId()+"\" ");
			sb.append("classIdRef=\""+(Math.abs(o.getIsA().getId()))+"\" ");
			
			
			// This is correct. The isArray field is not ever set in native code.
			boolean isArray = false;
						
			sb.append("isArray=\""+(isArray ? 1 : 0)+"\" ");
			sb.append("size=\""+o.getSize()+"\" ");
			
			TRCObjAllocSite as = o.getObjAllocSite();
			if(as  != null) {
				sb.append("aMethodIdRef=\""+as.getMethod().getId()+"\" ");
				sb.append("aLineNr=\""+as.getLineNumber()+"\" ");
			}

			String t = sb.toString();
			t = t.substring(0, t.length()-1) + "/>\n";
			
			os.write(t.getBytes());
		} else {
			sb.append("<objFree ");
			sb.append("time=\""+ad(o.getCollectTime(), initTime)+"\" ");
			sb.append("objIdRef=\""+o.getId()+"\" ");
			sb.append("age=\""+0+"\"/>\n"); // age value is not read/used by workbench 
			
			os.write(sb.toString().getBytes());
		}
	}
	
	public void outputRuntimeInit(OutputStream os, TRCNode node, TRCProcess process) throws IOException {
		String nodeTime = node.getAbsoluteTime() == null ? process.getAbsoluteTime().toPlainString() : node.getAbsoluteTime().toPlainString();
		
		StringBuffer sb = new StringBuffer();
		sb.append("<runtimeInitDone threadIdRef=\"1\" time=\""+nodeTime+"\"/>\n");
		os.write(sb.toString().getBytes());
	}

}


