/*******************************************************************************
 * Copyright (c) 2000, 2016 IBM Corporation, Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.trace.ui.internal.wizard;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.ui.internal.core.TraceFilterManager;
import org.eclipse.hyades.trace.ui.internal.core.TraceProfileFiltersUI;
import org.eclipse.hyades.trace.ui.internal.util.FilterSetElement;
import org.eclipse.hyades.trace.ui.internal.util.FilterTableElement;
import org.eclipse.hyades.trace.ui.internal.util.TraceMessages;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.WizardResourceImportPage;
import org.eclipse.ui.model.WorkbenchContentProvider;


public class ExportProfileSettingWizardPage extends WizardResourceImportPage{
	
	protected Text ProfileFilterSourceField;
	protected Button ProfileFilterSourceButton;
	protected int ExportType =0;
	protected FilterSetElement FSelement; 
	private Button Permission;

	
	public static final String  plainFilterExt = ".txt";
	public static final String fextension =  ".filterSet";
	public static final String PlanTXTFilterSeperator = "  ";
	
	/* The tags and attributes used to serialize the filters in XML */
	public static final String OPEN_FILTERS_TAG = "<filters>";
	public static final String CLOSE_FILTERS_TAG  = "</filters>";
	
	protected static String[] EXTs = {".xml", ".txt"};
	protected TraceProfileFiltersUI filterUI;
	
	public void setFSelement(FilterSetElement fSelement) {
		FSelement = fSelement;
	}

	public void setExportType(int ExportType) {
		this.ExportType = ExportType;
	}
	
	public TraceProfileFiltersUI getFilterUI() {
		return filterUI;
	}

	public void setFilterUI(TraceProfileFiltersUI filterUI) {
		this.filterUI = filterUI;
	}

	public ExportProfileSettingWizardPage(IStructuredSelection sel) {
		super(TraceMessages.EXPORT_WIZ_SUP_TITLE, sel);
		setTitle(TraceMessages.EXPORT_WIZ_TITLE);
		setDescription(TraceMessages.EXPORT_WIZ_TITLE);
		//this.selection = sel;
	}
	
	private Control[] createTextFieldWithBrowse(Composite parent,
			String labelName) {

		/* Create a new composite */
		Composite sourceContainerGroup = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		sourceContainerGroup.setLayout(layout);
		sourceContainerGroup.setFont(parent.getFont());
		sourceContainerGroup.setLayoutData(new GridData(
				GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
		

		/* Create the label */
		Label groupLabel = new Label(sourceContainerGroup, SWT.NONE);
		groupLabel.setText(labelName);
		groupLabel.setFont(parent.getFont());

		/* Create the Text object */
		Text sourceNameField = new Text(sourceContainerGroup, SWT.BORDER);
		GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.GRAB_HORIZONTAL);
		sourceNameField.setLayoutData(data);
		sourceNameField.setFont(parent.getFont());

		sourceNameField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				checkPageValid();

			}
		});

		/* Create the Browse button */
		Button sourceBrowseButton = new Button(sourceContainerGroup, SWT.PUSH);
		sourceBrowseButton
				.setText(TraceMessages.BROWSE2);
		sourceBrowseButton.addListener(SWT.Selection, this);
		GridData g = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		sourceBrowseButton.setLayoutData(g);
		sourceBrowseButton.setFont(parent.getFont());
		setButtonLayoutData(sourceBrowseButton);
		
		
		Composite addtional = new Composite(parent, SWT.NONE);
		GridLayout addtional_layout = new GridLayout();
		addtional_layout.numColumns = 2;
		addtional.setLayout(layout);
		addtional.setFont(parent.getFont());
		addtional.setLayoutData(new GridData(
				GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
		//TODO
		Button permissionbtn = new Button(addtional, SWT.CHECK);
		permissionbtn.setText(TraceMessages.EXPORT_WIZ_PERMISSION);
		permissionbtn.setData(false);
		permissionbtn.addListener(SWT.Selection, this);
		GridData g1 = new GridData(GridData.FILL);
		permissionbtn.setLayoutData(g1);
		permissionbtn.setFont(parent.getFont());
		
		
		return new Control[] { sourceNameField, sourceBrowseButton,permissionbtn };
	}
	
	@SuppressWarnings("restriction")
	@Override
	public void createControl(Composite composite) {

		/* Create a main composite */
		Composite coverageContainer = new Composite(composite, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		coverageContainer.setLayout(layout);
		coverageContainer.setFont(composite.getFont());
		coverageContainer.setLayoutData(new GridData(
				GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL
						| GridData.FILL_BOTH));

		/* Create the search boxes for the coveragedata files */
		Control[] widgets = createTextFieldWithBrowse(coverageContainer,
				TraceMessages.EXPORT_WIZ_PFF);
		ProfileFilterSourceField = (Text) widgets[0];
		ProfileFilterSourceButton = (Button) widgets[1];
		Permission = (Button) widgets[2];
		Permission.setVisible(false);
		
		ProfileFilterSourceField.setToolTipText(TraceMessages.EXPORT_FILTER_SOURCE_HELP);
		ProfileFilterSourceButton.setToolTipText(TraceMessages.EXPORT_FILTER_SOURCE_BRO_HELP);

		setControl(coverageContainer);

		PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, UIPlugin.getPluginId() + ".pfpse0002");

		setPageComplete(false);
	}
	
	@Override
	public void handleEvent(Event event) {

		if (event.widget == ProfileFilterSourceButton) {
			/* Handle the 'Browse' button on the coverage data textbox */
			handleSourceBrowseButtonPressed();
		} 
		if (event.widget == Permission){
			checkPageValid();
		}
	}
	
	protected void handleSourceBrowseButtonPressed() {
		FileDialog dialog = new FileDialog(ProfileFilterSourceField.getShell(),
				SWT.SAVE);
		String dataFileExtension = ExportProfileSettingWizardPage.EXTs[ExportType]; //$NON-NLS-1$

		StringBuffer filterName = new StringBuffer();
		filterName.append(dataFileExtension);
		
		dialog.setFilterNames(new String[] {filterName.toString()});
		dialog.setFilterExtensions(
				new String[] {"*" + dataFileExtension});
		dialog.setFileName(FSelement.getDisplayName());

		String selectedFile = dialog.open();
		if (selectedFile != null) {
			// If it is valid then proceed to populate
			setErrorMessage(null);
			ProfileFilterSourceField.setText(selectedFile);
			ProfileFilterSourceField.setFocus();
		}
	}
	
	
	protected boolean ensureSourceIsValid() {
		if (!new File(ProfileFilterSourceField.getText()).exists()) {
				return true;
		}
		return false;
	}
	
	protected boolean ensureSourceFolderIsValid(){
		File SourceFile =  new File(ProfileFilterSourceField.getText());
		return (!(SourceFile.getParentFile() == null)&& SourceFile.getParentFile().isDirectory());
	}
	
	protected boolean ensureSourceExtnesion(){
		if (ProfileFilterSourceField.getText().endsWith(EXTs[ExportType]))
			return true;
		return false;
	}
	
	private boolean checkPageValid() {
		if (!ensureSourceExtnesion()){
			setErrorMessage(TraceMessages.EXPORT_WIZ_FORMAT_ERR);
		}else if(!ensureSourceFolderIsValid()){
			setErrorMessage(TraceMessages.EXPORT_WIZ_DIR_ERR);
		}
			else if (!ensureSourceIsValid() && (!Permission.getSelection())) {
			setErrorMessage(TraceMessages.EXPORT_WIZ_FILE_EX_ERR);
			Permission.setVisible(true);
		}else {
			setErrorMessage(null);
			setPageComplete(true);
			return true;
		}
		setPageComplete(false);
		return false;
	}
	
	public boolean performFinish() {
		try {
			if (!ExportResource())
				return false;
		} catch (Exception e) {
			return false;
		}
		return true;
	}
	private boolean ExportResource() throws CoreException,
	InvocationTargetException, InterruptedException {
	
		
		File FilterFileToExport = getLaunchFileToExport();
		ArrayList<FilterSetElement> exportList= new ArrayList<FilterSetElement>();
		exportList.add(FSelement);
		exportFilterSetList(exportList, FilterFileToExport, this.ExportType);
		return true;
		 
	}
	
	/**
	 * Export and save targeted FilterSetElement items to local File system
	 * @param items a collection of FilterSetElements to target
	 * @param exportLocation the file location to save the export FilterSetElements
	 */
	public void exportFilterSetList(ArrayList<FilterSetElement> items, File exportFile, int option){
		try {
			//if filter already exists in FS, Ask user for permission to overwrite the old filter
			
			TraceFilterManager tf = TraceFilterManager.getInstance();
			exportFile.createNewFile();
			BufferedOutputStream outStream = new BufferedOutputStream(new FileOutputStream(exportFile));
			StringBuilder outputString = new StringBuilder();
			if (option == ExportProfileSettingOptionPage.TXTTYPE){
				for (FilterSetElement FSele: items){
					ArrayList<FilterTableElement> tempContainer = FSele.getChildren();
					for (FilterTableElement FTele: tempContainer){
						outputString.append(FTele.getText()).append(PlanTXTFilterSeperator)
						.append(FTele.getMethod()).append(PlanTXTFilterSeperator).
						append(FTele.getVisibility()).append('\n');
					}
					outStream.write((outputString.toString()).getBytes("UTF8"));//$NON-NLS-1$
				}
			}
			else if (option == ExportProfileSettingOptionPage.XMLTYPE){
				outputString.append(OPEN_FILTERS_TAG).append(TraceFilterManager.serializeFiltersToXML(items))
				.append(CLOSE_FILTERS_TAG);
			outStream.write((outputString.toString()).getBytes("UTF8"));//$NON-NLS-1$
			}
			outStream.flush();
			outStream.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private File getLaunchFileToExport(){
		return new File(ProfileFilterSourceField.getText());
	}
	

	


@Override
protected void createSourceGroup(Composite arg0) {
	// TODO Auto-generated method stub
	
}

@Override
protected ITreeContentProvider getFileProvider() {
	// TODO Auto-generated method stub
	return new WorkbenchContentProvider();
}

@Override
protected ITreeContentProvider getFolderProvider() {
	// TODO Auto-generated method stub
	return new WorkbenchContentProvider();
}
	
}
