/*******************************************************************************
 * Copyright (c) 2000, 2016 IBM Corporation, Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.trace.ui.internal.wizard;

import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.ui.internal.util.TraceMessages;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PlatformUI;



public class ImportProfileSettingOptionPage extends WizardPage{

	private static final String PAGE_NAME = TraceMessages.IMPORT_TYPES;
	private Button[] importTypes;
	public static final int XMLTYPE = 0;
	public static final int TXTTYPE = 1;
	
	public static final String SELECTION_SETTING_KEY = "Import profile filter Selection"; //$NON-NLS-1$

	private static final int DEFAULT_SELECTION = XMLTYPE;
	
	public ImportProfileSettingOptionPage() {
		super(PAGE_NAME);
		setTitle(TraceMessages.IMPORT_TYPE);
		setDescription(TraceMessages.IMPORT_TYPE_DES);
		
	}
	
	public ImportProfileSettingOptionPage(IStructuredSelection selection) {
		this();
	}
	
	
	public void createControl(Composite parent) {
		Composite mainComposite = new Composite(parent, SWT.NONE);
		mainComposite.setLayout(new GridLayout(2, false));
		mainComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		
		createPlaceHolder(mainComposite);

		
		importTypes = new Button[2];
		
		Button fileSystemRadio = new Button(mainComposite, SWT.RADIO);
		fileSystemRadio.setText(TraceMessages.IMPORT_XML);
		fileSystemRadio.setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,false));
		fileSystemRadio.setData("selection_type", new Integer(XMLTYPE)); //$NON-NLS-1$
		fileSystemRadio.setToolTipText(TraceMessages.XML_HELP);
		importTypes[0] = fileSystemRadio;
		
		createPlaceHolder(mainComposite);
		
		Button launchesRadio = new Button(mainComposite, SWT.RADIO);
		launchesRadio.setText(TraceMessages.IMPORT_TXT);
		launchesRadio.setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,false));
		launchesRadio.setData("selection_type", new Integer(TXTTYPE)); //$NON-NLS-1$
		launchesRadio.setToolTipText(TraceMessages.TXT_HELP);
		importTypes[1] = launchesRadio;
		
		
		IDialogSettings settings = fetchDialogSettings();
		int selectionType;
		if( settings != null && settings.get(SELECTION_SETTING_KEY) != null)
			 selectionType = settings.getInt(SELECTION_SETTING_KEY);
		else
			selectionType = DEFAULT_SELECTION;
		
		
		for (int i = 0; i < importTypes.length; i++) {
			Button option = importTypes[i];
			if (((Integer)option.getData("selection_type")).intValue()  //$NON-NLS-1$
					== selectionType) {
				option.setSelection(true);
			}
		}
		
		mainComposite.addDisposeListener(new DisposeListener() {

			public final void widgetDisposed(final DisposeEvent event) {
					getDialogSettings().put(SELECTION_SETTING_KEY, getSelectedImportType());
			}
		});
		
		setControl(mainComposite);
		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, UIPlugin.getPluginId() + ".pfpsi0001");
		
	}
	
	/**
	 * @return
	 */
	private IDialogSettings fetchDialogSettings() {
		return getWizard().getDialogSettings();
	}
	
	private void createPlaceHolder(Composite mainComposite) {
		Label blankLabel = new Label(mainComposite, SWT.NULL);
		blankLabel.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
	}

	/**
	 * @return
	 */
	public int getSelectedImportType() {
		for (int i = 0; i < importTypes.length; i++) {
			Button option = importTypes[i];
			if(option.getSelection()) {
				return ((Integer)option.getData("selection_type")).intValue(); //$NON-NLS-1$
			}
		}
		return -1;
		
	}
}
