/************************************************************************
* Copyright (c) 2017 IBM.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    IBM - Initial API and implementation
*    Anton Karputkin, IBM - Initial API and implementation
*
* $Id: NewThrowableEvent.h,v 1.0 2017-04-05 14:46:00 antonk Exp $
************************************************************************/

#include "ExceptionCatchEvent.h"
#include <cassert>
#include <list>

using namespace Martini::BaseProf;
using namespace Martini::CGProf;
using namespace Martini::MPI;

CExceptionCatchEvent::CExceptionCatchEvent() {}
CExceptionCatchEvent::~CExceptionCatchEvent() {}

using namespace std;
/*
* Init - initializes internal data and registers for leave method event
*/
TResult
CExceptionCatchEvent::Init(CProfEnv* profEnv)
{
	m_pProfEnv = profEnv;
	// register for leave method event
	TResult retVal = profEnv->m_pMpiApi->RegisterEvent(profEnv->m_clientId, *this);
	return retVal;
}

/*
* HandleEvent - callback function for leave method event
*/
void
CExceptionCatchEvent::HandleEvent(Martini::MPI::SExceptionCatchEventData &data)
{
	if (!m_pProfEnv->m_profilerIsActive) return;  //profiler is in pause mode, add for bug 289485
	LOG_ASSERT((data.validData & DR_THREAD_ID) != 0);
	LOG_ASSERT((data.validData & DR_THREAD_CPU_TIME) != 0);

	// Add for bug 194081
	//bool isContinueHandleMethodLeave = m_pProfEnv->m_pShadowStackTracker->TrackMethodLeave(data);
	//if (!isContinueHandleMethodLeave)
	//{
	//	return;
	//}
	// Add for bug 194081
	
	if (m_pProfEnv->ec_env->isCGExecDetails()) {
		list<SConstructorTimeEntry> finalizedEntries;
		m_pProfEnv->m_Tickets.FinalizeConstructorEntries(data.threadId, data.constructorCalls, data.constructorCallsLength, finalizedEntries);
//TODO: Remove it when they finally support this
#if defined(IBM_COMPILER_NO_CPP11_SUPPORT)
		for(list<SConstructorTimeEntry>::iterator entryIter = finalizedEntries.begin(); entryIter != finalizedEntries.end(); ++entryIter){
			SConstructorTimeEntry &entry = *entryIter;
#else
		for (auto &entry : finalizedEntries) {
#endif
			SMethodInfo classInfo;
			TResult iRes = m_pProfEnv->m_pMpiApi->GetMethodInfo(m_pProfEnv->m_clientId, entry.methodId,
				DR_CLASS_ID, &classInfo);
			LOG_ASSERT(iRes == MRTE_RESULT_OK);
			LOG_ASSERT((classInfo.validData & DR_CLASS_ID) != 0);
			m_pProfEnv->ec_env->PrintMethodExitElement(data.threadId, entry.methodId, classInfo.classId, entry.ticket, entry.time);
		}
		
	}
	else {
		m_pProfEnv->FinalizeStackEntries(data.threadId, data.constructorCalls, data.constructorCallsLength);
	}
}

BitSet
CExceptionCatchEvent::EventDataTypes()
{
	return  DR_THREAD_ID | DR_THREAD_ELAPSED_TIME | DR_THREAD_CPU_TIME;
}
