/************************************************************************
* Copyright (c) 2017 IBM.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    IBM - Initial API and implementation
*    Anton Karputkin, IBM - Initial API and implementation
*
* $Id: NewThrowableEvent.h,v 1.0 2017-03-31 14:23:42 antonk Exp $
************************************************************************/

#include "NewThrowableEvent.h"
#include <iostream>
#include <assert.h>
using namespace Martini::BaseProf;
using namespace Martini::CGProf;
using namespace Martini::MPI;

CNewThrowableEvent::CNewThrowableEvent() {}
CNewThrowableEvent::~CNewThrowableEvent() {}

/*
* Init - initializes internal data and registers for leave method event
*/
TResult
CNewThrowableEvent::Init(CProfEnv* profEnv)
{
	m_pProfEnv = profEnv;
	// register for leave method event
	TResult retVal = profEnv->m_pMpiApi->RegisterEvent(profEnv->m_clientId, *this);
	return retVal;
}

/*
* HandleEvent - callback function for leave method event
*/
void
CNewThrowableEvent::HandleEvent(Martini::MPI::SNewThrowableEventData &data)
{
	if (!m_pProfEnv->m_profilerIsActive) return;  //profiler is in pause mode, add for bug 289485
	LOG_ASSERT((data.validData & DR_THREAD_ID) != 0);
	LOG_ASSERT((data.validData & DR_THREAD_CPU_TIME) != 0);
	//static bool b = false;
	//assert(b);
	//b = true;
	if (m_pProfEnv->ec_env->isCGExecDetails()) {
		//profiler is active
		m_pProfEnv->m_Tickets.UpdateConstructorEntries(data.threadId, data.constructorCalls, data.constructorCallsLength, data.uiCpuNanos);
	}
	else {
		m_pProfEnv->UpdateStackEntries(data.threadId, data.constructorCalls, data.constructorCallsLength, data.uiCpuNanos);
	}
	//delete[] data.constructorCalls;
}

BitSet
CNewThrowableEvent::EventDataTypes()
{
	return  DR_THREAD_ID | DR_THREAD_ELAPSED_TIME | DR_THREAD_CPU_TIME;
}
